
#include <glib.h>
#include <gtk/gtk.h>

#include "actions.h"
#include "ammp.h"
#include "cdaudio_stream.h"
#include "common.h"
#include "mpeg_stream.h"
#include "music_stream.h"
#include "support.h"





static GtkWidget *file_selector = NULL;
static const gchar *list_item_data_key = "list_item_data";



// Prototypes
void widgets_sensitivity (gboolean sensitive);
void store_selections (GtkWidget *button, gpointer data);
void destroy_file_selector (GtkWidget *button, gpointer data);
void cd_browser (GtkWidget *button, gpointer data);
void add_track (GtkWidget *button, GtkList *list);

gchar *media_name (gchar *filename);
gint media_get_type (gchar *filename);




// Files actions
void media_browser (GtkWidget *button, gpointer data)
{
  GtkWidget *probe_cd;

  if (file_selector)
    destroy_file_selector (NULL, NULL);

  file_selector = gtk_file_selection_new (_("Media browser"));

  // FIXME : I can't pass window object through signal media_browser...
  // gtk_widget_set_parent (file_selector, GTK_WIDGET (GTK_WINDOW (data)));
  // gtk_window_set_position (GTK_WINDOW (file_selector), GTK_WIN_POS_CENTER_ON_PARENT);

  gtk_file_selection_show_fileop_buttons (GTK_FILE_SELECTION (file_selector));
  gtk_file_selection_set_select_multiple (GTK_FILE_SELECTION (file_selector), TRUE);


  
  // Check if any medium was inserted
  if (!use_cdrom)
  {
    cd_init (&cdrom);
    checkCDMounted();
    memset (&cdrom.thiscd, 0, sizeof (cdrom.thiscd));

    use_cdrom = cd_status (&cdrom);

    if ((cdrom.cur_cdmode == DISC_PLAYING || cdrom.cur_cdmode == DISC_PAUSED) && use_cdrom)
      cd_stop (&cdrom);

    cdrom.cur_track = 1;
  }

  // Add probe_cd button if use_cdrom is not FALSE
  if (use_cdrom)
  {
    probe_cd = gtk_button_new_with_mnemonic (_("Probe CD"));
    g_signal_connect (G_OBJECT (probe_cd), "clicked",
                      G_CALLBACK (cd_browser), data);
    gtk_box_pack_start (GTK_BOX (GTK_FILE_SELECTION (file_selector)->main_vbox),
                        probe_cd,
                        TRUE, TRUE, 8);
    gtk_widget_show (probe_cd);
  }

  g_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (file_selector)->ok_button),
                    "clicked",
                    G_CALLBACK (store_selections),
                    (gpointer) file_selector);

  g_signal_connect_swapped (GTK_FILE_SELECTION (file_selector)->cancel_button,
			                      "clicked",
		                        G_CALLBACK (destroy_file_selector),
				                    (gpointer) file_selector);

  gtk_widget_show (file_selector);
}

void store_selections (GtkWidget *button, gpointer data)
{
  gint i, is_new;
  gchar **selections;

  selections = gtk_file_selection_get_selections (GTK_FILE_SELECTION (data));

  destroy_file_selector (NULL, NULL);

  if (medias_list == NULL)
  {
    is_new = 1;
    medias_list = g_ptr_array_sized_new (G_N_ELEMENTS (selections));
  }
  else
  {
    is_new = 0;
    g_ptr_array_set_size (medias_list, medias_list->len + G_N_ELEMENTS (selections) - 1);
  }

  for (i = 0; selections[i] != NULL; i++)
    g_ptr_array_add (medias_list, selections[i]);

  if (is_new)
  {
    media_num = (i == 1 ? 0 : 1);
    media_type = media_get_type (g_ptr_array_index (medias_list, media_num));

    gtk_label_set_text (GTK_LABEL (media_title_label),
                        media_name (g_ptr_array_index (medias_list, media_num)));
    widgets_sensitivity (TRUE);
  }
}

void destroy_file_selector (GtkWidget *button, gpointer data)
{
    gtk_widget_destroy (file_selector);
    file_selector = NULL;
    return;
}


void cd_browser (GtkWidget *button, gpointer data)
{
  GtkWidget *cd_dialog = NULL;
  GtkWidget *vbox;
  GtkWidget *list, *list_item;
  GtkWidget *scrolled_win;
  GtkWidget *add_button;

  gchar *buffer;
  gint cd_is_open, i;


  destroy_file_selector (NULL, NULL);

  cd_dialog = gtk_dialog_new_with_buttons (g_strdup_printf (_("CD Browser"),
                                          SDL_CDName (0)),
                                          GTK_WINDOW (data),
                                          GTK_DIALOG_DESTROY_WITH_PARENT,
                                          NULL,
                                          GTK_RESPONSE_NONE,
                                          NULL);

  gtk_window_set_resizable (GTK_WINDOW (cd_dialog), FALSE);
  gtk_container_set_border_width (GTK_CONTAINER (cd_dialog), 8);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (GTK_DIALOG (cd_dialog)->action_area),
                            GTK_BUTTONBOX_EDGE);

  vbox = gtk_vbox_new (FALSE, 8);
  gtk_container_add (GTK_CONTAINER (GTK_DIALOG (cd_dialog)->vbox), vbox);


  if (!use_cdrom)
  {
    gtk_box_pack_start (GTK_BOX (vbox),
                        gtk_label_new_with_mnemonic (g_strdup_printf (
                        _("Sorry, I think there's no\nmedium in %s"), cd_device)),
                        TRUE, TRUE, 8);
  }
  else
  {
    gtk_box_pack_start (GTK_BOX (vbox),
                        gtk_label_new_with_mnemonic (
                        _("Select track then add it to playlist")),
                        TRUE, TRUE, 4);

    scrolled_win = gtk_scrolled_window_new (NULL, NULL);
    gtk_container_set_border_width (GTK_CONTAINER (scrolled_win), 5);
    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_win),
                                    GTK_POLICY_AUTOMATIC,
                                    GTK_POLICY_AUTOMATIC);
    gtk_box_pack_start (GTK_BOX (vbox),
                        scrolled_win,
                        TRUE, TRUE, 8);

    list = gtk_list_new();
    gtk_list_set_selection_mode (GTK_LIST (list), GTK_SELECTION_BROWSE);
    gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (scrolled_win), list);

    for (i = 0; i < cdrom.thiscd.ntracks; i++)
    {
      buffer = g_strdup_printf ("%s %2.2d", TRACK_WORD, i + 1);
      list_item = gtk_list_item_new_with_label (buffer);
      gtk_object_set_data (GTK_OBJECT (list_item), list_item_data_key, buffer);
      gtk_container_add (GTK_CONTAINER (list), list_item);
    }

    add_button = gtk_button_new_with_mnemonic (_("Add selection"));
    g_signal_connect (G_OBJECT (add_button), "clicked",
                      G_CALLBACK (add_track), list);
    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (cd_dialog)->action_area),
                        add_button,
                        TRUE, TRUE, 4);
  }

  gtk_dialog_add_button (GTK_DIALOG (cd_dialog), _("Close"), GTK_RESPONSE_NONE);


  gtk_widget_show_all (cd_dialog);
  gtk_dialog_run (GTK_DIALOG (cd_dialog));
  gtk_widget_destroy (cd_dialog);
}

// Add a selected track to playlist
void add_track (GtkWidget *button, GtkList *list)
{
  GList *tracks_list;
  GtkObject *list_item;
  gchar *item_data_string;

  gint i, is_new;

  tracks_list = GTK_LIST (list)->selection;

  if (!tracks_list)
    return;

	list_item = GTK_OBJECT (tracks_list->data);
	item_data_string = gtk_object_get_data (list_item, list_item_data_key);

  if (medias_list == NULL)
  {
    is_new = 1;
    medias_list = g_ptr_array_sized_new (2);
  }
  else
  {
    is_new = 0;
    g_ptr_array_set_size (medias_list, medias_list->len);
  }

  g_ptr_array_add (medias_list, item_data_string);

  if (is_new)
  {
    media_num = 0;
    media_type = media_get_type (g_ptr_array_index (medias_list, media_num));

    gtk_label_set_text (GTK_LABEL (media_title_label),
                        media_name (g_ptr_array_index (medias_list, media_num)));
    widgets_sensitivity (TRUE);
  }
}


// Action on media init
gchar *media_name (gchar *filename)
{
  gchar *name;
  gchar *bufptr;

  bufptr = g_strrstr (filename, "/");
  if (bufptr != NULL)
    name = g_strdup (&filename[bufptr - filename + 1]);
  else
    name = g_strdup (filename);

  return (g_strndup (name, 30));
}


gint track_num (void)
{
  gchar *track_name;
  gchar *bufptr;

  track_name = g_strdup (g_ptr_array_index (medias_list, media_num));

  bufptr = g_strrstr (track_name, g_strdup_printf (" ", TRACK_WORD));

  return (atoi (&track_name[bufptr - track_name + 1]) - 1);
}


/* Return the type of the media that we want to
 * play. FIXME :
 * If there's an error, we use "media theory"
 * ;-)... MEDIA_TYPE * MEDIA_ERROR : it means that
 * if a file is called test.mp3.wave (we supposed
 * it's a wave stream), ammp will play this file
 * like a mp3 stream with an incertainty, sorry. */
gint media_get_type (gchar *filename)
{
  gchar *name, *bufptr;
  gint len;

  len = g_strlen (filename);

  bufptr = g_strrstr (filename, g_strdup_printf ("%s ", TRACK_WORD));
  if (bufptr != NULL)
  {
    cd_status (&cdrom);
    cdrom.cur_track = track_num();
    set_cd_time (cdrom.thiscd.trk[track_num()].length);

    if ((bufptr - filename) == 0)
      return (MEDIA_IS_CDA);
    else
      return (MEDIA_IS_CDA * MEDIA_ERROR);
  }


  name = g_strdup (g_ascii_strdown (filename, len));

  bufptr = g_strrstr (name, ".mod");
  if (bufptr != NULL)
  {
    if (len - (bufptr - name) == 4)
      return (MEDIA_IS_MOD);
    else
      return (MEDIA_IS_MOD * MEDIA_ERROR);
  }

  bufptr = g_strrstr (name, ".mp3");
  if (bufptr != NULL)
  {
    if (!load_mp3 (filename))
      return;

    set_time ((gint) mp3_info.total_time);

    if (len - (bufptr - name) == 4)
      return (MEDIA_IS_MP3);
    else
      return (MEDIA_IS_MP3 * MEDIA_ERROR);
  }

  bufptr = g_strrstr (name, ".mpg");
  if (bufptr != NULL)
  {
    if (!load_mpg (filename))
      return;

    set_time ((gint) mpg_info.total_time);

    if (len - (bufptr - name) == 4)
      return (MEDIA_IS_MPG);
    else
      return (MEDIA_IS_MPG * MEDIA_ERROR);
  }

  bufptr = g_strrstr (name, ".mpeg");
  if (bufptr != NULL)
  {
    if (!load_mpg (filename))
      return;

    set_time ((gint) mpg_info.total_time);

    if (len - (bufptr - name) == 4)
      return (MEDIA_IS_MPG);
    else
      return (MEDIA_IS_MPG * MEDIA_ERROR);
  }

  bufptr = g_strrstr (name, ".wav");
  if (bufptr != NULL)
  {
    if (len - (bufptr - name) == 4)
      return (MEDIA_IS_WAV);
    else
      return (MEDIA_IS_WAV * MEDIA_ERROR);
  }

  bufptr = g_strrstr (name, ".wave");
  if (bufptr != NULL)
  {
    if (len - (bufptr - name) == 5)
      return (MEDIA_IS_WAV);
    else
      return (MEDIA_IS_WAV * MEDIA_ERROR);
  }

  return (MEDIA_ERROR);
}


// prev, play, pause, stop, next buttons & items
void widgets_sensitivity (gboolean sensitive)
{
  GtkWidget *item_widget;

  item_widget = gtk_item_factory_get_widget (item_factory, _("<main>/Go/Previous"));
  gtk_widget_set_sensitive (item_widget, sensitive);

  item_widget = gtk_item_factory_get_widget (item_factory, _("<main>/Go/Play"));
  gtk_widget_set_sensitive (item_widget, sensitive);

  item_widget = gtk_item_factory_get_widget (item_factory, _("<main>/Go/Pause"));
  gtk_widget_set_sensitive (item_widget, sensitive);

  item_widget = gtk_item_factory_get_widget (item_factory, _("<main>/Go/Stop"));
  gtk_widget_set_sensitive (item_widget, sensitive);

  item_widget = gtk_item_factory_get_widget (item_factory, _("<main>/Go/Next"));
  gtk_widget_set_sensitive (item_widget, sensitive);

  gtk_widget_set_sensitive (prev_button, sensitive);
  gtk_widget_set_sensitive (play_button, sensitive);
  gtk_widget_set_sensitive (pause_button, sensitive);
  gtk_widget_set_sensitive (stop_button, sensitive);
  gtk_widget_set_sensitive (next_button, sensitive);
  gtk_widget_set_sensitive (playlist_button, sensitive);
}




// Medias actions
void previous_media (GtkWidget *button, gpointer data)
{
  gint type;

  if (timer)
    g_source_remove (timer);

  gtk_label_set_text (GTK_LABEL (time_label), "--:--");
  gtk_adjustment_set_value (GTK_ADJUSTMENT (progress_adj), 0.0);

  type = media_type;

  media_num  = (media_num - 1 < 0 ? medias_list->len - 1 : media_num - 1);

  if (type != media_get_type (g_ptr_array_index (medias_list, media_num)) && is_playing)
  {
    stop_media (NULL, NULL);
    is_playing = 1;
  }

  media_type = media_get_type (g_ptr_array_index (medias_list, media_num));

  gtk_label_set_text (GTK_LABEL (media_title_label),
                        media_name (g_ptr_array_index (medias_list, media_num)));

  if (is_playing)
    play_media (NULL, NULL);
}


void play_media (GtkWidget *button, gpointer data)
{
  switch (media_type)
  {
    case MEDIA_IS_CDA :
    {
      cd_status (&cdrom);
      switch (cdrom.cur_cdmode)
      {
        case DISC_PLAYING:
        {
          if (!is_playing)
            cd_stop (&cdrom);
        }
        break;
      }

      cdrom.cur_track = track_num() + 1;
      cd_play (&cdrom, cdrom.cur_track, 0, cdrom.cur_track + 1);
      is_playing = 1;
    }
    break;

    case MEDIA_IS_MOD :
    {

    }
    break;

    case MEDIA_IS_MP3 :
    {
      // Ensure that mp3 stream is on its own beginning
      SMPEG_getinfo (mp3, &mp3_info);
      if ((gint) mp3_info.current_time != 0)
      {
        SMPEG_stop (mp3);
        load_mp3 (g_ptr_array_index (medias_list, media_num));
      }

      play_mp3();
      is_playing = 1;
    }
    break;

    case MEDIA_IS_MPG :
    {
      // Ensure that mpg stream is on its own beginning
      SMPEG_getinfo (mpg, &mpg_info);
      if ((gint) mpg_info.current_time != 0)
      {
        SMPEG_stop (mpg);
        load_mpg (g_ptr_array_index (medias_list, media_num));
      }

      play_mpg();
      is_playing = 1;
    }
    break;

    case MEDIA_IS_WAV :
    {

    }
    break;


    // User  select is own demuxer strategy
    case MEDIA_ERROR :
    {

    }
    break;
  }

  GTK_ADJUSTMENT (progress_adj)->upper = 100.0;
  gtk_adjustment_set_value (GTK_ADJUSTMENT (progress_adj), 0.0);

  if (timer)
    g_source_remove (timer);

  timer = g_timeout_add (TIMER_PERIOD, progress_timeout, NULL);
}


void pause_media (GtkWidget *button, gpointer data)
{
  switch (media_type)
  {
    case MEDIA_IS_CDA :
    {
      cd_status (&cdrom);
      switch (cdrom.cur_cdmode)
      {
        case DISC_PLAYING :
        {
          is_playing = 0;
          cd_pause (&cdrom);
          g_source_remove (timer);
        }
        break;

        case DISC_PAUSED :
        {
          is_playing = 1;
          cd_pause (&cdrom);
          timer = g_timeout_add (TIMER_PERIOD, progress_timeout, NULL);
        }
        break;
      }
    }
    break;

    case MEDIA_IS_MOD :
    {

    }
    break;

    case MEDIA_IS_MP3 :
    {
      if (mp3 != NULL)
      {
        if (SMPEG_status (mp3) == SMPEG_PLAYING)
        {
          SMPEG_pause (mp3);
          is_playing = 0;
          g_source_remove (timer);
        }
        else if (SMPEG_status (mp3) == SMPEG_STOPPED)
        {
          SMPEG_pause (mp3);
          is_playing = 1;
          timer = g_timeout_add (TIMER_PERIOD, progress_timeout, NULL);
        }
      }
    }
    break;

    case MEDIA_IS_MPG :
    {
      if (mpg != NULL)
      {
        if (SMPEG_status (mpg) == SMPEG_PLAYING)
        {
          SMPEG_pause (mpg);
          is_playing = 0;
          g_source_remove (timer);
        }
        else if (SMPEG_status (mpg) == SMPEG_STOPPED)
        {
          SMPEG_pause (mpg);
          is_playing = 1;
          timer = g_timeout_add (TIMER_PERIOD, progress_timeout, NULL);
        }
      }
    }
    break;

    case MEDIA_IS_WAV :
    {

    }
    break;


    // User  select is own demuxer strategy
    case MEDIA_ERROR :
    {

    }
    break;
  }
}

void stop_media (GtkWidget *button, gpointer data)
{
  is_playing = 0;

  switch (media_type)
  {
    case MEDIA_IS_CDA :
    {
      cd_stop (&cdrom);
      set_cd_time (cdrom.thiscd.trk[track_num()].length);
    }
    break;

    case MEDIA_IS_MOD :
    {

    }
    break;

    case MEDIA_IS_MP3 :
    {
      if (mp3 != NULL)
      {
        if (SMPEG_status (mp3) != SMPEG_ERROR)
        {
          SMPEG_stop (mp3);
          set_time ((gint) mp3_info.total_time);
        }
      }
    }
    break;

    case MEDIA_IS_MPG :
    {
     if (mpg != NULL)
      {
        if (SMPEG_status (mpg) != SMPEG_ERROR)
        {
          stop_mpg();
          set_time ((gint) mpg_info.total_time);
        }
      }
    }
    break;

    case MEDIA_IS_WAV :
    {

    }
    break;


    // User  select is own demuxer strategy
    case MEDIA_ERROR :
    {

    }
    break;
  }

  if (timer)
    g_source_remove (timer);

  gtk_adjustment_set_value (GTK_ADJUSTMENT (progress_adj), 0.0);
  GTK_ADJUSTMENT (progress_adj)->upper = 0.0;
}


void next_media (GtkWidget *button, gpointer data)
{
  gint type;

  if (timer)
    g_source_remove (timer);

  gtk_label_set_text (GTK_LABEL (time_label), "--:--");
  gtk_adjustment_set_value (GTK_ADJUSTMENT (progress_adj), 0.0);

  media_num  = (media_num + 1 == medias_list->len ? 0 : media_num + 1);

  if (type != media_get_type (g_ptr_array_index (medias_list, media_num)) && is_playing)
  {
    stop_media (NULL, NULL);
    is_playing = 1;
  }

  media_type = media_get_type (g_ptr_array_index (medias_list, media_num));

  gtk_label_set_text (GTK_LABEL (media_title_label),
                        media_name (g_ptr_array_index (medias_list, media_num)));

  if (is_playing)
    play_media (NULL, NULL);
}


void set_time (gint time_factor)
{
  m = (time_factor / 60) % 60;
  s = time_factor % 60;
  gtk_label_set_text (GTK_LABEL (time_label), g_strdup_printf ("%2.2d:%2.2d", m, s));
}


void set_cd_time (gint time_factor)
{
  m = time_factor / 60;
  s = time_factor % 60;
  gtk_label_set_text (GTK_LABEL (time_label), g_strdup_printf ("%2.2d:%2.2d", m, s));
}
