
#include <glib.h>
#include <gdk/gdk.h>
#include <gtk/gtk.h>


#include "about.h"
#include "actions.h"
#include "common.h"
#include "playlist.h"
#include "support.h"



// Prototypes
void quit_request (GtkWidget *window, gpointer data);
void exit_request (void);
GtkWidget *gtk_button_new_with_pixmaps (const gchar *filename);
void progress_adj_changed (GtkAdjustment *adj, gpointer data);
void hscale_drag_on (GtkWidget *widget, gpointer data);
void hscale_drag_off (GtkWidget *widget, gpointer data);
gint progress_timeout (gpointer data);


// The main application
GtkWidget *ammp_run (void)
{
  GtkWidget *window = NULL;
  GtkWidget *main_table, *hbox;

  GdkColor color = {0, 0xffff, 0, 0};
  PangoFontDescription *font_desc;


  GtkAccelGroup *accel_group;
  GtkItemFactoryEntry menu_items[] =
  {
    {_("/_File"),		        NULL,	        0,		          0, "<Branch>"                   },
    {_("/File/_Open"),	    "<control>O", media_browser,  0, "<StockItem>", GTK_STOCK_OPEN},
    {_("/File/_Playlist"),	"<control>P", menu_playlist,  0, "<StockItem>", GTK_STOCK_JUSTIFY_LEFT},
    {_("/File/sep1"),       NULL,         0,              0, "<Separator>"                                                                                     },
    {_("/File/_Quit"),	    "<control>Q", quit_request,	  0, "<StockItem>", GTK_STOCK_QUIT},

    {_("/_Go"),             NULL,         0,              0, "<Branch>"                   },
    {_("/Go/P_revious"),    NULL,         previous_media, 0, "<Item>"                     },
    {_("/Go/_Play"),        NULL,         play_media,     0, "<Item>"                     },
    {_("/Go/P_ause"),       NULL,         pause_media,    0, "<Item>"                     },
    {_("/Go/_Stop"),        NULL,         stop_media,     0, "<Item>"                     },
    {_("/Go/_Next"),        NULL,         next_media,     0, "<Item>"                     },

    {_("/_Help"),		        NULL,	        0,		          0, "<LastBranch>"               },
    {_("/Help/_About"),	    "<control>A",	menu_about,	    0, "<StockItem>", GTK_STOCK_HELP},
  };


  GtkWidget *tbar;
  GtkWidget *quit_button, *about_button, *open_button;



  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (window), PROGRAM);
  gtk_window_set_resizable (GTK_WINDOW (window), FALSE);
  gtk_container_set_border_width (GTK_CONTAINER (window), 0);
  g_signal_connect (G_OBJECT (window), "destroy", G_CALLBACK (quit_request), NULL);


  main_table = gtk_table_new (4, 4, FALSE);
  gtk_container_add (GTK_CONTAINER (window), main_table);



  // Create menubar (from item factory)

  accel_group = gtk_accel_group_new();
  item_factory = gtk_item_factory_new (GTK_TYPE_MENU_BAR, "<main>", accel_group);
      g_object_set_data_full (G_OBJECT (window), "<main>",
			      item_factory, (GDestroyNotify) g_object_unref);
  gtk_window_add_accel_group (GTK_WINDOW (window), accel_group);
  gtk_item_factory_create_items (item_factory, G_N_ELEMENTS (menu_items),
                                menu_items, (gpointer) window);


  gtk_table_attach (GTK_TABLE (main_table),
                    gtk_item_factory_get_widget (item_factory, "<main>"),
                    0, 1, 0, 1,
                    GTK_FILL | GTK_EXPAND, 0, 0, 0);



  // Create toolbar

  tbar = gtk_toolbar_new();


  quit_button = gtk_button_new_with_pixmaps ("quit.png");
  g_signal_connect (G_OBJECT (quit_button), "clicked",
                    G_CALLBACK (quit_request), NULL);


  prev_button = gtk_button_new_with_pixmaps ("prev.png");
  g_signal_connect (G_OBJECT (prev_button), "clicked",
                    G_CALLBACK (previous_media), NULL);


  play_button = gtk_button_new_with_pixmaps ("play.png");
  g_signal_connect (G_OBJECT (play_button), "clicked",
                    G_CALLBACK (play_media), NULL);


  pause_button = gtk_button_new_with_pixmaps ("pause.png");
  g_signal_connect (G_OBJECT (pause_button), "clicked",
                    G_CALLBACK (pause_media), NULL);


  stop_button = gtk_button_new_with_pixmaps ("stop.png");
  g_signal_connect (G_OBJECT (stop_button), "clicked",
                    G_CALLBACK (stop_media), NULL);


  next_button = gtk_button_new_with_pixmaps ("next.png");
  g_signal_connect (G_OBJECT (next_button), "clicked",
                    G_CALLBACK (next_media), NULL);


  open_button = gtk_button_new_with_pixmaps ("open.png");
  g_signal_connect (G_OBJECT (open_button), "clicked",
                    G_CALLBACK (media_browser), (gpointer) window);
                    
  
  playlist_button = gtk_button_new_with_pixmaps ("playlist.png");
  g_signal_connect (G_OBJECT (playlist_button), "clicked",
                    G_CALLBACK (open_playlist), (gpointer) window);


  about_button = gtk_button_new_with_pixmaps ("about.png");
  g_signal_connect (G_OBJECT (about_button), "clicked",
                    G_CALLBACK (ammp_about), (gpointer) window);


  gtk_toolbar_append_widget (GTK_TOOLBAR (tbar), quit_button, NULL, _("Quit"));
  gtk_toolbar_append_widget (GTK_TOOLBAR (tbar), gtk_vseparator_new(), NULL, NULL);
  gtk_toolbar_append_widget (GTK_TOOLBAR (tbar), prev_button, NULL, _("Previous"));
  gtk_toolbar_append_widget (GTK_TOOLBAR (tbar), play_button, NULL, _("Play"));
  gtk_toolbar_append_widget (GTK_TOOLBAR (tbar), pause_button, NULL, "Pause");
  gtk_toolbar_append_widget (GTK_TOOLBAR (tbar), stop_button, NULL, _("Stop"));
  gtk_toolbar_append_widget (GTK_TOOLBAR (tbar), next_button, NULL, _("Next"));
  gtk_toolbar_append_widget (GTK_TOOLBAR (tbar), gtk_vseparator_new(), NULL, NULL);
  gtk_toolbar_append_widget (GTK_TOOLBAR (tbar), open_button, NULL, _("Open"));
  gtk_toolbar_append_widget (GTK_TOOLBAR (tbar), playlist_button, NULL, _("Playlist"));
  gtk_toolbar_append_widget (GTK_TOOLBAR (tbar), gtk_vseparator_new(), NULL, NULL);
  gtk_toolbar_append_widget (GTK_TOOLBAR (tbar), about_button, NULL, _("About"));


  gtk_table_attach (GTK_TABLE (main_table),
                    tbar,
                    0, 1, 1, 2,
                    GTK_FILL | GTK_EXPAND, 0, 0, 0);



  // Set adjustment to inactivity mode
  progress_adj = (GtkAdjustment *) gtk_adjustment_new (0.0, 0.0, 0.0, 1.0, 0.0, 0.0);
  g_signal_connect (G_OBJECT (progress_adj), "value_changed",
                    G_CALLBACK (progress_adj_changed), NULL);
  
  progress_hscale = gtk_hscale_new (GTK_ADJUSTMENT (progress_adj));
  hscale_dragging = 0;

  gtk_range_set_update_policy (GTK_RANGE (progress_hscale), GTK_UPDATE_CONTINUOUS);
  gtk_scale_set_draw_value (GTK_SCALE (progress_hscale), FALSE);
  g_signal_connect (G_OBJECT (progress_hscale), "button_press_event",
                    G_CALLBACK (hscale_drag_on), (gpointer) window);
  g_signal_connect (G_OBJECT (progress_hscale), "button_release_event",
                    G_CALLBACK (hscale_drag_off), (gpointer) window);
  gtk_table_attach (GTK_TABLE (main_table),
                    progress_hscale,
                    0, 4, 2, 3,
                    GTK_EXPAND | GTK_FILL, 0, 20, 40);




  // Some labels...
  hbox = gtk_hbox_new (0, FALSE);
  gtk_table_attach (GTK_TABLE (main_table),
                    hbox,
                    0, 4, 3, 4,
                    GTK_FILL | GTK_EXPAND, 0, 2, 2);


  time_label = gtk_label_new_with_mnemonic ("--:--");
  font_desc = pango_font_description_from_string ("Helvetica,Sans Bold 12");
  gtk_widget_modify_font (time_label, font_desc);
  gtk_widget_modify_fg (time_label, GTK_STATE_NORMAL, &color);
  gtk_box_pack_start (GTK_BOX (hbox),
                      time_label, FALSE, FALSE, 2);

  media_title_label = gtk_label_new_with_mnemonic (_("No media selected"));
  gtk_box_pack_start (GTK_BOX (hbox),
                      media_title_label, FALSE, FALSE, 12);


  // turn off some widgets sensitivity
  widgets_sensitivity (FALSE);

  return (window);
}



void quit_request (GtkWidget *widget, gpointer data)
{
  if ((cdrom.cur_cdmode == DISC_PLAYING || cdrom.cur_cdmode == DISC_PAUSED) && use_cdrom)
    cd_stop (&cdrom);

  gtk_main_quit();
  exit_request();
}


void exit_request (void)
{
  Mix_CloseAudio();
  SDL_Quit();
}



GtkWidget *gtk_button_new_with_pixmaps (const gchar *filename)
{
  GtkWidget *button;
  GtkWidget *pixmaps;


  button = gtk_button_new();
  gtk_container_set_border_width (GTK_CONTAINER (button), 2);

  pixmaps = create_pixmap (filename);

  gtk_container_add (GTK_CONTAINER (button), pixmaps);

  return (button);
}



void progress_adj_changed (GtkAdjustment *adj, gpointer data)
{
  if (!hscale_dragging)
    return;

  switch (media_type)
  {
    case MEDIA_IS_CDA :
    {
      cd_status (&cdrom);
      cd_play (&cdrom, cdrom.cur_track,
              (gint) ((cdrom.thiscd.trk[track_num()].length * (gint) adj->value) / 100),
              cdrom.cur_track + 1);

      cd_status (&cdrom);
      set_cd_time (cdrom.cur_pos_rel);
    }
    break;

    case MEDIA_IS_MOD :
    {

    }
    break;

    case MEDIA_IS_MP3 :
    {
      if (SMPEG_status (mp3) != SMPEG_ERROR)
      {
        SMPEG_getinfo (mp3, &mp3_info);
        SMPEG_seek (mp3, (gint) ((mp3_info.total_size * (gint) adj->value) / 100));

        SMPEG_getinfo (mp3, &mp3_info);
        set_time ((gint) mp3_info.current_time);
      }
    }
    break;

    case MEDIA_IS_MPG :
    {
      if (SMPEG_status (mpg) != SMPEG_ERROR)
      {
        SMPEG_getinfo (mpg, &mpg_info);
        SMPEG_seek (mpg, (gint) ((mpg_info.total_size * (gint) adj->value) / 100));

        SMPEG_getinfo (mpg, &mpg_info);
        set_time ((gint) mpg_info.current_time);
      }
    }
    break;

    case MEDIA_IS_WAV :
    {

    }
    break;
  }
}

void hscale_drag_on (GtkWidget *widget, gpointer data)
{
  hscale_dragging = 1;

  /* FIXME : I don't know why progress_hscale is not able to
   * be dragging without this useless line */
  g_print ("");
}


void hscale_drag_off (GtkWidget *widget, gpointer data)
{
  hscale_dragging = 0;

  /* FIXME : I don't know why progress_hscale is not able to
   * be dragging without this useless line */
  g_print ("");
}


gint progress_timeout (gpointer data)
{
  if (hscale_dragging)
    return (TRUE);

  switch (media_type)
  {
    case MEDIA_IS_CDA :
    {
      cd_status (&cdrom);
      /*if (cdrom.cur_pos_rel == cdrom.thiscd.trk[track_num()].length - 2)
        next_media (NULL, NULL);*/
      if (cdrom.cur_cdmode == DISC_PLAYING)
      {
        set_cd_time (cdrom.cur_pos_rel);
        gtk_adjustment_set_value (GTK_ADJUSTMENT (progress_adj),
                                  (gdouble) (100. * cdrom.cur_pos_rel /
                                  cdrom.thiscd.trk[track_num()].length));

        return (TRUE);
      }
      else
        next_media (NULL, NULL);
    }
    break;

    case MEDIA_IS_MOD :
    {

    }
    break;

    case MEDIA_IS_MP3 :
    {
      if (SMPEG_status (mp3) == SMPEG_PLAYING)
      {
        SMPEG_getinfo (mp3, &mp3_info);
        set_time ((gint) mp3_info.current_time);

        gtk_adjustment_set_value (GTK_ADJUSTMENT (progress_adj),
                                  (gdouble) (100. * mp3_info.current_time /
                                  mp3_info.total_time));

        return (TRUE);
      }
      else
        next_media (NULL, NULL);
    }
    break;

    case MEDIA_IS_MPG :
    {
      if (SMPEG_status (mpg) == SMPEG_PLAYING)
      {
        SMPEG_getinfo (mpg, &mpg_info);
        set_time ((gint) mpg_info.current_time);

        gtk_adjustment_set_value (GTK_ADJUSTMENT (progress_adj),
                                  (gdouble) (100. * mpg_info.current_time /
                                  mpg_info.total_time));

        return (TRUE);
      }
      else
        next_media (NULL, NULL);
    }
    break;

    case MEDIA_IS_WAV :
    {
    
    }
    break;
  }
}

