
#include <glib.h>

#include <SDL/SDL.h>
#include <smpeg/smpeg.h>

#include "common.h"
#include "support.h"




gint load_mp3 (gchar *filename)
{
  if (mp3 != NULL)
  {
    SMPEG_delete (mp3);
    mp3 = NULL;
  }

  mp3 = SMPEG_new (filename, &mp3_info, 1);
  if (SMPEG_error (mp3))
  {
    g_print (_("The stream %s can't be loaded :\n%s\n\n"),
            filename, SMPEG_error (mp3));
    return (0);
  }

  return (1);
}


void play_mp3 (void)
{
  SMPEG_getinfo (mp3, &mp3_info);

  if (mp3_info.has_audio)
    SMPEG_play (mp3);
}



void update (SDL_Surface *screen, Sint32 x, Sint32 y, Uint32 w, Uint32 h)
{
  if (screen->flags & SDL_DOUBLEBUF)
    SDL_Flip(screen);
}



gint load_mpg (gchar *filename)
{
  if (mpg != NULL)
  {
    SDL_FreeSurface (screen);
    screen = NULL;

    SMPEG_delete (mpg);
    mpg = NULL;
  }

  mpg = SMPEG_new (filename, &mpg_info, 1);
  if (SMPEG_error (mpg))
  {
    g_print (_("The stream %s can't be loaded :\n%s\n\n"),
            filename, SMPEG_error (mpg));
    return (0);
  }

  return (1);
}


gint play_mpg (void)
{
  SDL_VideoInfo *video_info;

  SMPEG_getinfo (mpg, &mpg_info);

  if (mpg_info.has_audio || mpg_info.has_video)
  {
    if (screen == NULL)
    {
      /* Get the "native" video mode */
      video_info = (SDL_VideoInfo *) SDL_GetVideoInfo();
      screen = SDL_SetVideoMode (mpg_info.width * VIDEO_SCALE, mpg_info.height * VIDEO_SCALE,
                                video_info->vfmt->BitsPerPixel,
                                SDL_DOUBLEBUF | SDL_HWSURFACE);
      if (screen == NULL)
      {
        g_print (_("\nCouldn't set SDL video mode :\n%s\n"), SDL_GetError());
        return (0);
      }

      SDL_WM_SetCaption (g_strdup_printf (_("%s - video output"), PACKAGE), NULL);
      SDL_ShowCursor (0);
      SMPEG_setdisplay (mpg, screen, NULL, update);
      SMPEG_scaleXY (mpg, screen->w, screen->h);
    }

    SMPEG_play (mpg);
  }
}


void stop_mpg (void)
{
  if (mpg != NULL)
  {
    SDL_FreeSurface (screen);
    screen = NULL;

    SMPEG_delete (mpg);
    mpg = NULL;
    
    SDL_Quit();
    
    if (SDL_Init (SDL_INIT_AUDIO | SDL_INIT_VIDEO) < 0)
      g_print (_("\nCan't initialize SDL in AUDIO, VIDEO, and CDROM mode :\n%s\n"),
        SDL_GetError());
  }
}
